@@ -19,6 +19,14 @@ extern caddr_t mmap __P((caddr_t __addr, size_t __len,
 extern int munmap __P((caddr_t __addr, size_t __len));
 extern int mprotect __P ((caddr_t __addr, size_t __len, int __prot));
 
+extern int msync __P((caddr_t __addr, size_t __len, int __flags));
+
+extern int mlock __P((caddr_t __addr, size_t __len));
+extern int munlock __P((caddr_t __addr, size_t __len));
+
+extern int mlockall __P((int __flags));
+extern int munlockall __P((void));
+
 __END_DECLS
 
 #endif /* _SYS_MMAN_H */
