@@ -68,7 +68,7 @@ rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 	pid = getpid();
 #endif
 	hp = gethostbyname(*ahost);
-	if (hp == 0 || sizeof (sin.sin_addr) < hp->h_length) {
+	if (hp == 0) {
 #if NLS
 		fprintf(stderr, "%s: %s\n", *ahost,
                               catgets(_libc_cat, HerrorListSet,
@@ -301,7 +301,7 @@ ruserok(const char *rhost, int superuser, const char *ruser,
 	const char *luser)
 {
 	FILE *hostf;
-	char *fhost;
+	char fhost[MAXHOSTNAMELEN];
 	int first = 1;
 	register const char *sp;
 	register char *p;
@@ -310,7 +310,6 @@ ruserok(const char *rhost, int superuser, const char *ruser,
 
 	saveuid = geteuid();
 	sp = rhost;
-	fhost = alloca (strlen (rhost) + 1);
 	p = fhost;
 	while (*sp) {
 		if (*sp == '.') {
@@ -335,13 +334,11 @@ again:
 	if (first == 1) {
 		struct stat sbuf;
 		struct passwd *pwd;
-		char *pbuf;
+		char pbuf[MAXPATHLEN];
 
 		first = 0;
 		if ((pwd = getpwnam(luser)) == NULL)
 			return(-1);
-		pbuf = alloca (strlen (pwd->pw_dir)
-			+ sizeof ("/.rhosts") + 1);
 		(void)strcpy(pbuf, pwd->pw_dir);
 		(void)strcat(pbuf, "/.rhosts");
 		(void)seteuid(pwd->pw_uid);
@@ -393,56 +390,36 @@ _validuser(FILE *hostf, const char *rhost, const char *luser,
 			user = p;
 		*p = '\0';
 #ifdef YP
-	uservalid = hostvalid = 0;
             /* disable host from -hostname entry */
         if ('-' == ahost[0] && '@' != ahost[1]
             && _checkhost(rhost, &ahost[1], baselen))
-          hostvalid = 1;
+          return -1;
             /* disable host from -@netgroup entry for host */
         if ('-' == ahost[0] && '@' == ahost[1] && '\0' != ahost[2]
             && _checknetgrouphost(rhost, &ahost[2], baselen))
-          hostvalid = 1;
+          return -1;
             /* disable user from -user entry */
         if ('\0' != *user && user[0] == '-' && user[1] != '@'
             && !strcmp(&user[1], ruser))
-          uservalid = 1;
+          return -1;
             /* disable user from -@netgroup entry for user */
         if ('\0' != *user && user[0] == '-' && user[1] == '@'
             && user[2] != '\0' && _checknetgroupuser(ruser, &user[2]))
-          uservalid = 1;
-	if (hostvalid && (uservalid || *user == '\0'))
-	  return -1;
-
-	if ('+' == ahost[0] && '\0' == ahost[1])
-	  /* enable all hosts if + entry for host */
-	  hostvalid = 1;
-	else if ('+' == ahost[0] && '@' == ahost[1] && '\0' != ahost[2])
-          /* enable host from +@netgroup entry for host */
+          return -1;
+            /* enable host from +@netgroup entry for host */
+        if ('+' == ahost[0] && '@' == ahost[1] && '\0' != ahost[2])
           hostvalid = _checknetgrouphost(rhost, &ahost[2], baselen);
         else
           hostvalid = _checkhost(rhost, ahost, baselen);
-
             /* enable user from +@netgroup entry for user */
-	/* -@netgroup entry already gave uservalid=1 */
-	if (user[0] == '+')
-	{
-	  if (user[1] == '\0')
-	    /* enable all users if + entry for user */
-	    uservalid = 1;
-	  else if (user[1] == '@' && user[2] != '\0')
-	    uservalid = _checknetgroupuser(ruser, &user[2]);
-	  else uservalid = !strcmp(&user[1], ruser);
-	}
-	else if (user[0] == '-')
-	  uservalid = -uservalid;
-	else
-	  uservalid = !strcmp(ruser, *user ? user : luser);
+        if ('\0' != *user && user[0] == '+'
+            && user[1] == '@' && user[2] != '\0')
+          uservalid = _checknetgroupuser(ruser, &user[2]);
+        else
+          uservalid = !strcmp(ruser, *user ? user : luser);
         
-        if (hostvalid)
-	  if (uservalid == 1)
-	    return 0;
-	else if (uservalid == -1)
-	  return (-1);
+        if (hostvalid && uservalid)
+          return 0;
 #else
 		if (_checkhost(rhost, ahost, baselen) &&
 		    !strcmp(ruser, *user ? user : luser)) {
@@ -459,6 +436,7 @@ _checkhost(const char *rhost, const char *lhost, int len)
 	static char ldomain[MAXHOSTNAMELEN + 1];
 	static char *domainp = NULL;
 	static int nodomain = 0;
+	static struct hostent *hp;
 	register char *cp;
 
 	if (len == -1)
@@ -472,13 +450,18 @@ _checkhost(const char *rhost, const char *lhost, int len)
 	if (nodomain)
 		return(0);
 	if (!domainp) {
-		if (getdomainname(ldomain, sizeof(ldomain)) == -1) {
+		if (gethostname(ldomain, sizeof(ldomain)) == -1) {
 			nodomain = 1;
 			return(0);
 		}
 		ldomain[MAXHOSTNAMELEN] = (char) 0;
-		domainp = ldomain;
-		for (cp = domainp; *cp; ++cp)
+		if ((hp = gethostbyname(ldomain)) == NULL)
+		        return(0);
+		if ((domainp = index(hp->h_name, '.')) == (char *)NULL) {
+			nodomain = 1;
+			return(0);
+		}
+		for (cp = ++domainp; *cp; ++cp)
 			if (isupper(*cp))
 				*cp = tolower(*cp);
 	}
@@ -498,16 +481,19 @@ _checknetgrouphost(const char *rhost, const char *netgr, int baselen)
   setnetgrent(netgr);
   while (1)
     {
-      status = getnetgrent(&host, &user, &domain);
+      while (1 == (status = getnetgrent(&host, &user, &domain))
+             && NULL == host
+             && NULL != domain
+             && 0 != strcmp(domain, nisdomain))
+        ;  /* find valid host entry */
       
-      if (0 == status)
+      if (0 == status || NULL == host)
         {
           endnetgrent();
           return 0;
         }
 
-      if((domain == NULL || !strcmp(domain, nisdomain))
-	 && (host == NULL || _checkhost(rhost, host, baselen)))
+      if(1 == _checkhost(rhost, host, baselen))
         {
           endnetgrent();
           return 1;
@@ -527,16 +513,19 @@ _checknetgroupuser(const char *ruser, const char *netgr)
   setnetgrent(netgr);
   while (1)
     {
-      status = getnetgrent(&host, &user, &domain);
-
-      if (0 == status)
+      while (1 == (status = getnetgrent(&host, &user, &domain))
+             && NULL == user
+             && NULL != domain
+             && 0 != strcmp(domain, nisdomain))
+        ;  /* find valid user entry */
+      
+      if (0 == status || NULL == user)
         {
           endnetgrent();
           return 0;
         }
 
-      if((domain == NULL || 0 == strcmp(domain, nisdomain))
-	 && (user == NULL || 0 == strcmp(ruser, user)))
+      if(0 == strcmp(ruser, user))
         {
           endnetgrent();
           return 1;
