@@ -1,5 +1,5 @@
 /* 
-Copyright (C) 1993 Free Software Foundation
+Copyright (C) 1993, 1995 Free Software Foundation
 
 This file is part of the GNU IO Library.  This library is free
 software; you can redistribute it and/or modify it under the
@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -30,8 +30,9 @@ the executable file might be covered by the GNU General Public License. */
 #endif
 #include <string.h>
 
-void _IO_un_link(fp)
-     _IO_FILE *fp;
+void
+DEFUN(_IO_un_link, (fp),
+      _IO_FILE *fp)
 {
   if (fp->_flags & _IO_LINKED) {
     _IO_FILE **f;
@@ -45,8 +46,9 @@ void _IO_un_link(fp)
   }
 }
 
-void _IO_link_in(fp)
-     _IO_FILE *fp;
+void
+DEFUN(_IO_link_in, (fp),
+      _IO_FILE *fp)
 {
     if ((fp->_flags & _IO_LINKED) == 0) {
 	fp->_flags |= _IO_LINKED;
@@ -59,8 +61,8 @@ void _IO_link_in(fp)
    Assumes the current get area is the main get area. */
 
 _IO_size_t
-_IO_least_marker(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_least_marker, (fp),
+      register _IO_FILE *fp)
 {
   _IO_ssize_t least_so_far = fp->_IO_read_end - fp->_IO_read_base;
   register struct _IO_marker *mark;
@@ -73,8 +75,8 @@ _IO_least_marker(fp)
 /* Switch current get area from backup buffer to (start of) main get area. */
 
 void
-_IO_switch_to_main_get_area(fp)
-     _IO_FILE *fp;
+DEFUN(_IO_switch_to_main_get_area, (fp),
+      _IO_FILE *fp)
 {
   char *tmp;
   fp->_flags &= ~_IO_IN_BACKUP;
@@ -88,8 +90,8 @@ _IO_switch_to_main_get_area(fp)
 /* Switch current get area from main get area to (end of) backup area. */
 
 void
-_IO_switch_to_backup_area(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_switch_to_backup_area, (fp),
+     register _IO_FILE *fp)
 {
   char *tmp;
   fp->_flags |= _IO_IN_BACKUP;
@@ -101,11 +103,11 @@ _IO_switch_to_backup_area(fp)
 }
 
 int
-_IO_switch_to_get_mode(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_switch_to_get_mode, (fp),
+     register _IO_FILE *fp)
 {
   if (fp->_IO_write_ptr > fp->_IO_write_base)
-    if (fp->_jumps->__overflow(fp, EOF) == EOF)
+    if (_IO_OVERFLOW (fp, EOF) == EOF)
       return EOF;
   if (_IO_in_backup(fp))
     fp->_IO_read_base = fp->_IO_backup_base;
@@ -124,8 +126,8 @@ _IO_switch_to_get_mode(fp)
 }
 
 void
-_IO_free_backup_area(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_free_backup_area, (fp),
+     register _IO_FILE *fp)
 {
   if (_IO_in_backup (fp))
     _IO_switch_to_main_get_area(fp);  /* Just in case. */
@@ -137,8 +139,8 @@ _IO_free_backup_area(fp)
 
 #if 0
 int
-_IO_switch_to_put_mode(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_switch_to_put_mode, (fp),
+      register _IO_FILE *fp)
 {
   fp->_IO_write_base = fp->_IO_read_ptr;
   fp->_IO_write_ptr = fp->_IO_read_ptr;
@@ -153,16 +155,16 @@ _IO_switch_to_put_mode(fp)
 }
 #endif
 
-int  __overflow(f, ch)
-     _IO_FILE *f;
-     int ch;
+int
+DEFUN(__overflow, (f, ch),
+      _IO_FILE *f AND int ch)
 {
-  return f->_jumps->__overflow(f, ch);
+  return _IO_OVERFLOW (f, ch);
 }
 
 static int
-save_for_backup (fp)
-     _IO_FILE *fp;
+DEFUN(save_for_backup, (fp),
+      _IO_FILE *fp)
 {
   /* Append [_IO_read_base.._IO_read_end] to backup area. */
   int least_mark = _IO_least_marker(fp);
@@ -224,8 +226,8 @@ save_for_backup (fp)
 }
 
 int
-__underflow(fp)
-     _IO_FILE *fp;
+DEFUN(__underflow, (fp),
+      _IO_FILE *fp)
 {
   if (_IO_in_put_mode(fp))
     if (_IO_switch_to_get_mode(fp) == EOF) return EOF;
@@ -244,12 +246,12 @@ __underflow(fp)
     }
   else if (_IO_have_backup(fp))
     _IO_free_backup_area(fp);
-  return fp->_jumps->__underflow(fp);
+  return _IO_UNDERFLOW (fp);
 }
 
 int
-__uflow(fp)
-     _IO_FILE *fp;
+DEFUN(__uflow, (fp),
+     _IO_FILE *fp)
 {
   if (_IO_in_put_mode(fp))
     if (_IO_switch_to_get_mode(fp) == EOF) return EOF;
@@ -268,15 +270,12 @@ __uflow(fp)
     }
   else if (_IO_have_backup(fp))
     _IO_free_backup_area(fp);
-  return fp->_jumps->__uflow(fp);
+  return _IO_UFLOW (fp);
 }
 
 void
-_IO_setb(f, b, eb, a)
-     _IO_FILE *f;
-     char *b;
-     char *eb;
-     int a;
+DEFUN(_IO_setb, (f, b, eb, a),
+      _IO_FILE *f AND char *b AND char *eb AND int a)
 {
   if (f->_IO_buf_base && !(f->_flags & _IO_USER_BUF))
     FREE_BUF(f->_IO_buf_base);
@@ -289,41 +288,39 @@ _IO_setb(f, b, eb, a)
 }
 
 void
-_IO_doallocbuf(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_doallocbuf, (fp),
+      register _IO_FILE *fp)
 {
   if (fp->_IO_buf_base)
     return;
   if (!(fp->_flags & _IO_UNBUFFERED))
-    if (fp->_jumps->__doallocate(fp) != EOF)
+    if (_IO_DOALLOCATE (fp) != EOF)
       return;
   _IO_setb(fp, fp->_shortbuf, fp->_shortbuf+1, 0);
 }
 
 int
-_IO_default_underflow (fp)
-     _IO_FILE *fp;
+DEFUN(_IO_default_underflow, (fp),
+      _IO_FILE *fp)
 {
   return EOF;
 }
 
 int
-_IO_default_uflow (fp)
-     _IO_FILE *fp;
+DEFUN(_IO_default_uflow, (fp),
+      _IO_FILE *fp)
 {
-  int ch = fp->_jumps->__underflow(fp);
+  int ch = _IO_UNDERFLOW (fp);
   if (ch == EOF)
     return EOF;
   return *(unsigned char*)fp->_IO_read_ptr++;
 }
 
 _IO_size_t
-_IO_default_xsputn(f, data, n)
-     register _IO_FILE *f;
-     const void *data;
-     _IO_size_t n;
+DEFUN(_IO_default_xsputn, (f, data, n),
+      register _IO_FILE *f AND const void *data AND _IO_size_t n)
 {
-  register const char *s = data;
+  register const char *s = (char*) data;
   register _IO_size_t more = n;
   if (more <= 0)
     return 0;
@@ -359,23 +356,19 @@ _IO_default_xsputn(f, data, n)
 }
 
 _IO_size_t
-_IO_sgetn(fp, data, n)
-     _IO_FILE *fp;
-     void *data;
-     _IO_size_t n;
+DEFUN(_IO_sgetn, (fp, data, n),
+      _IO_FILE *fp AND void *data AND _IO_size_t n)
 {
   /* FIXME handle putback buffer here! */
-  return fp->_jumps->__xsgetn(fp, data, n);
+  return _IO_XSGETN (fp, data, n);
 }
 
 _IO_size_t
-_IO_default_xsgetn(fp, data, n)
-     _IO_FILE *fp;
-     void *data;
-     _IO_size_t n;
+DEFUN(_IO_default_xsgetn, (fp, data, n),
+      _IO_FILE *fp AND void *data AND _IO_size_t n)
 {
   register _IO_size_t more = n;
-  register char *s = data;
+  register char *s = (char*) data;
   for (;;)
     {
       _IO_ssize_t count = fp->_IO_read_end - fp->_IO_read_ptr; /* Data available. */
@@ -407,22 +400,18 @@ _IO_default_xsgetn(fp, data, n)
 }
 
 int
-_IO_sync(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_sync, (fp),
+      register _IO_FILE *fp)
 {
-  if (fp->_IO_read_ptr == fp->_IO_read_end && fp->_IO_write_ptr == fp->_IO_write_base)
-    return 0;
-  return EOF;
+  return 0;
 }
 
-int
-_IO_default_setbuf(fp, p, len)
-     register _IO_FILE *fp;
-     char* p;
-     _IO_ssize_t len;
+_IO_FILE*
+DEFUN(_IO_default_setbuf, (fp, p, len),
+      register _IO_FILE *fp AND char* p AND _IO_ssize_t len)
 {
-    if (fp->_jumps->__sync(fp) == EOF)
-	return EOF;
+    if (_IO_SYNC (fp) == EOF)
+	return NULL;
     if (p == NULL || len == 0)
       {
 	fp->_flags |= _IO_UNBUFFERED;
@@ -435,21 +424,19 @@ _IO_default_setbuf(fp, p, len)
       }
     fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_write_end = 0;
     fp->_IO_read_base = fp->_IO_read_ptr = fp->_IO_read_end = 0;
-    return 0;
+    return fp;
 }
 
 _IO_pos_t
-_IO_default_seekpos(fp, pos, mode)
-     _IO_FILE *fp;
-     _IO_pos_t pos;
-     _IO_seekflags mode;
+DEFUN(_IO_default_seekpos, (fp, pos, mode),
+      _IO_FILE *fp AND _IO_pos_t pos AND int mode)
 {
-  return fp->_jumps->__seekoff(fp, _IO_pos_as_off(pos), mode);
+  return _IO_SEEKOFF (fp, _IO_pos_as_off(pos), 0, mode);
 }
 
 int
-_IO_default_doallocate(fp)
-     _IO_FILE *fp;
+DEFUN(_IO_default_doallocate, (fp),
+      _IO_FILE *fp)
 {
   char *buf = ALLOC_BUF(_IO_BUFSIZ);
   if (buf == NULL)
@@ -459,9 +446,8 @@ _IO_default_doallocate(fp)
 }
 
 void
-_IO_init(fp, flags)
-     register _IO_FILE *fp;
-     int flags;
+DEFUN(_IO_init, (fp, flags),
+      register _IO_FILE *fp AND int flags)
 {
   fp->_flags = _IO_MAGIC|flags;
   fp->_IO_buf_base = NULL;
@@ -479,11 +465,12 @@ _IO_init(fp, flags)
   fp->_IO_save_end = NULL;
   fp->_markers = NULL;
   fp->_cur_column = 0;
+  fp->_fileno = -1;
 }
 
 int
-_IO_default_sync (fp)
-     _IO_FILE *fp;
+DEFUN(_IO_default_sync, (fp),
+      _IO_FILE *fp)
 {
   return 0;
 }
@@ -491,8 +478,9 @@ _IO_default_sync (fp)
 /* The way the C++ classes are mapped into the C functions in the
    current implementation, this function can get called twice! */
 
-void _IO_default_finish (fp)
-     _IO_FILE *fp;
+void
+DEFUN(_IO_default_finish, (fp),
+      _IO_FILE *fp)
 {
   struct _IO_marker *mark;
   if (fp->_IO_buf_base && !(fp->_flags & _IO_USER_BUF))
@@ -514,18 +502,15 @@ void _IO_default_finish (fp)
 }
 
 _IO_pos_t
-_IO_default_seekoff(fp, offset, mode)
-     register _IO_FILE *fp;
-     _IO_off_t offset;
-     _IO_seekflags mode;
+DEFUN(_IO_default_seekoff, (fp, offset, dir, mode),
+      register _IO_FILE *fp AND _IO_off_t offset AND int dir AND int mode)
 {
     return _IO_pos_BAD;
 }
 
 int
-_IO_sputbackc(fp, c) /* ungetc(c, fp) */
-     register _IO_FILE *fp;
-     int c;
+DEFUN(_IO_sputbackc, (fp, c),
+      register _IO_FILE *fp AND int c)
 {
   if (fp->_IO_read_ptr > fp->_IO_read_base
       && (unsigned char)fp->_IO_read_ptr[-1] == (unsigned char)c)
@@ -533,12 +518,12 @@ _IO_sputbackc(fp, c) /* ungetc(c, fp) */
       fp->_IO_read_ptr--;
       return (unsigned char)c;
     }
-  return fp->_jumps->__pbackfail(fp, c);
+  return _IO_PBACKFAIL (fp, c);
 }
 
 int
-_IO_sungetc(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_sungetc, (fp),
+      register _IO_FILE *fp)
 {
   if (fp->_IO_read_ptr > fp->_IO_read_base)
     {
@@ -546,25 +531,23 @@ _IO_sungetc(fp)
       return (unsigned char)*fp->_IO_read_ptr;
     }
   else
-    return fp->_jumps->__pbackfail(fp, EOF);
+    return _IO_PBACKFAIL (fp, EOF);
 }
 
 #if 0 /* Work in progress */
 void
-_IO_set_column(fp, c)
-     register _IO_FILE *fp;
-     int c;
+DEFUN(_IO_set_column, (fp, c),
+      register _IO_FILE *fp AND int c)
 {
   if (c == -1)
-    fp->_collumn = -1;
+    fp->_column = -1;
   else
-    fp->_collumn = c - (fp->_IO_write_ptr - fp->_IO_write_base);
+    fp->_column = c - (fp->_IO_write_ptr - fp->_IO_write_base);
 }
 #else
 int
-_IO_set_column(fp, i)
-     register _IO_FILE *fp;
-     int i;
+DEFUN(_IO_set_column, (fp, i),
+      register _IO_FILE *fp AND int i)
 {
   fp->_cur_column = i+1;
   return 0;
@@ -573,10 +556,8 @@ _IO_set_column(fp, i)
 
 
 unsigned
-_IO_adjust_column(start, line, count)
-     unsigned start;
-     const char *line;
-     int count;
+DEFUN(_IO_adjust_column, (start, line, count),
+      unsigned start AND const char *line AND int count)
 {
   register const char *ptr = line + count;
   while (ptr > line)
@@ -586,8 +567,8 @@ _IO_adjust_column(start, line, count)
 }
 
 int
-_IO_get_column(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_get_column, (fp),
+      register _IO_FILE *fp)
 {
   if (fp->_cur_column) 
     return _IO_adjust_column(fp->_cur_column - 1,
@@ -597,38 +578,42 @@ _IO_get_column(fp)
 }
 
 int
-_IO_flush_all()
+DEFUN_VOID(_IO_flush_all)
 {
   int result = 0;
   _IO_FILE *fp;
+
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
     if (fp->_IO_write_ptr > fp->_IO_write_base
-	&& fp->_jumps->__overflow(fp, EOF) == EOF)
+	&& _IO_OVERFLOW (fp, EOF) == EOF)
       result = EOF;
   return result;
 }
 
 void
-_IO_flush_all_linebuffered()
+DEFUN_VOID(_IO_flush_all_linebuffered)
 {
   _IO_FILE *fp;
+
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
     if (fp->_flags & _IO_LINE_BUF)
-      fp->_jumps->__overflow(fp, EOF);
+      _IO_OVERFLOW (fp, EOF);
 }
 
 void
-_IO_unbuffer_all ()
+DEFUN_VOID(_IO_unbuffer_all)
 {
   _IO_FILE *fp;
+
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
     if (! (fp->_flags & _IO_UNBUFFERED))
-      fp->_jumps->__setbuf(fp, NULL, 0);
+      _IO_SETBUF (fp, NULL, 0);
 }
 
 void
-_IO_cleanup ()
+DEFUN_VOID(_IO_cleanup)
 {
+  __libc_lock_lock (__libc_libio_lock);
   _IO_flush_all ();
 
   /* We currently don't have a reliable mechanism for making sure that
@@ -639,11 +624,12 @@ _IO_cleanup ()
      The following will make the standard streambufs be unbuffered, 
      which forces any output from late destructors to be written out. */
   _IO_unbuffer_all ();
+  __libc_lock_unlock (__libc_libio_lock);
 }
 
-void _IO_init_marker(marker, fp)
-     struct _IO_marker *marker;
-     _IO_FILE *fp;
+void
+DEFUN(_IO_init_marker, (marker, fp),
+      struct _IO_marker *marker AND _IO_FILE *fp)
 {
   marker->_sbuf = fp;
   if (_IO_in_put_mode(fp))
@@ -659,8 +645,8 @@ void _IO_init_marker(marker, fp)
 }
 
 void
-_IO_remove_marker (marker)
-     register struct _IO_marker * marker;
+DEFUN(_IO_remove_marker, (marker),
+      register struct _IO_marker *marker)
 {
   /* Unlink from sb's chain. */
   register struct _IO_marker **ptr = &marker->_sbuf->_markers;
@@ -683,17 +669,16 @@ _IO_remove_marker (marker)
 #define BAD_DELTA EOF
 
 int
-_IO_marker_difference(mark1, mark2)
-     struct _IO_marker *mark1;
-     struct _IO_marker *mark2;
+DEFUN(_IO_marker_difference, (mark1, mark2),
+      struct _IO_marker *mark1 AND struct _IO_marker *mark2)
 {
   return mark1->_pos - mark2->_pos;
 }
 
 /* Return difference between MARK and current posistion of MARK's stream. */
 int
-_IO_marker_delta(mark)
-     struct _IO_marker *mark;
+DEFUN(_IO_marker_delta, (mark),
+      struct _IO_marker *mark)
 {
   int cur_pos;
   if (mark->_sbuf == NULL)
@@ -705,10 +690,9 @@ _IO_marker_delta(mark)
   return mark->_pos - cur_pos;
 }
 
-int _IO_seekmark(fp, mark, delta)
-     _IO_FILE *fp;
-     struct _IO_marker *mark;
-     int delta;
+int
+DEFUN(_IO_seekmark, (fp, mark, delta),
+      _IO_FILE *fp AND struct _IO_marker *mark AND int delta)
 {
   if (mark->_sbuf != fp)
     return EOF;
@@ -727,8 +711,9 @@ int _IO_seekmark(fp, mark, delta)
   return 0;
 }
 
-void _IO_unsave_markers(fp)
-     register _IO_FILE *fp;
+void
+DEFUN(_IO_unsave_markers, (fp),
+     register _IO_FILE *fp)
 {
   register struct _IO_marker *mark = fp->_markers;
   if (mark)
@@ -755,9 +740,8 @@ void _IO_unsave_markers(fp)
 }
 
 int
-_IO_nobackup_pbackfail(fp, c)
-     register _IO_FILE *fp;
-     int c;
+DEFUN(_IO_nobackup_pbackfail, (fp, c),
+     register _IO_FILE *fp AND int c)
 {
   if (fp->_IO_read_ptr > fp->_IO_read_base)
 	fp->_IO_read_ptr--;
@@ -767,9 +751,8 @@ _IO_nobackup_pbackfail(fp, c)
 }
 
 int
-_IO_default_pbackfail(fp, c)
-     register _IO_FILE *fp;
-     int c;
+DEFUN(_IO_default_pbackfail, (fp, c),
+      register _IO_FILE *fp AND int c)
 {
   if (fp->_IO_read_ptr <= fp->_IO_read_base)
       {
@@ -814,36 +797,29 @@ _IO_default_pbackfail(fp, c)
 }
 
 _IO_pos_t
-_IO_default_seek(fp, offset, dir)
-     _IO_FILE *fp;
-     _IO_off_t offset;
-     int dir;
+DEFUN(_IO_default_seek, (fp, offset, dir),
+      _IO_FILE *fp AND _IO_off_t offset AND int dir)
 {
   return _IO_pos_BAD;
 }
 
 int
-_IO_default_stat(fp, st)
-     _IO_FILE *fp;
-     void* st;
+DEFUN(_IO_default_stat, (fp, st),
+      _IO_FILE *fp AND void* st)
 {
   return EOF;
 }
 
 _IO_ssize_t
-_IO_default_read(fp, data, n)
-     register _IO_FILE* fp;
-     void* data;
-     _IO_ssize_t n;
+DEFUN(_IO_default_read, (fp, data, n),
+      register _IO_FILE* fp AND void* data AND _IO_ssize_t n)
 {
   return -1;
 }
 
 _IO_ssize_t
-_IO_default_write(fp, data, n)
-     register _IO_FILE* fp;
-     const void* data;
-     _IO_ssize_t n;
+DEFUN(_IO_default_write, (fp, data, n),
+      register _IO_FILE* fp AND const void* data AND _IO_ssize_t n)
 {
   return 0;
 }
@@ -870,6 +846,6 @@ __io_defs io_defs__;
 #if defined(__ELF__) || defined(__GNU_LIBRARY__)
 #include <gnu-stabs.h>
 #ifdef weak_alias
-weak_alias (_IO_flush_all, _cleanup);
+weak_alias (_IO_cleanup, _cleanup);
 #endif
 #endif
