@@ -126,14 +126,12 @@ static unsigned int guess_grouping (unsigned int intdig_max,
 static char *group_number (char *buf, char *bufend, unsigned int intdig_no,
 			   const char *grouping, wchar_t thousands_sep);
 
-int __printf_fp( FILE *, const struct printf_info *, const ** );
-static unsigned int guess_grouping( unsigned int, const char *, wchar_t );
-static char *group_number( char *, char *, unsigned int,
-								  const char *, wchar_t );
-char hack_digit __P (( void ));
 
 int
-__printf_fp( FILE *fp, const struct printf_info *info, const **const args )
+__printf_fp (fp, info, args)
+     FILE *fp;
+     const struct printf_info *info;
+     const **const args;
 {
   /* The floating-point value to output.  */
   union
@@ -159,7 +157,7 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
   /* We need to shift the contents of fp_input by this amount of bits.	*/
   int to_shift;
 
-  /* The significant of the floating-point value in question  */
+  /* The significant of the floting-point value in question  */
   MPN_VAR(frac);
   /* and the exponent.	*/
   int exponent;
@@ -186,9 +184,6 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
   /* General helper (carry limb).  */
   mp_limb cy;
 
-  /* This is supposed to be the way to prototype a nested function. vch */
-  auto char hack_digit (void);
-
   char hack_digit (void)
     {
       mp_limb hi;
@@ -227,7 +222,7 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
 	}
 
       return '0' + hi;
-    }	/* end of 'hack_digit()' */
+    }
 
 
   /* Figure out the decimal point character.  */
@@ -239,11 +234,7 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
   if (info->group)
     {
       grouping = _NL_CURRENT (LC_NUMERIC, GROUPING);
-#ifdef __CHAR_UNSIGNED__
-      if (*grouping == CHAR_MAX )
-#else
-      if (*grouping == CHAR_MAX || *grouping < 0)
-#endif
+      if (*grouping <= 0 || *grouping == CHAR_MAX)
 	grouping = NULL;
       else
 	{
@@ -258,7 +249,6 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
   else
     grouping = NULL;
 
-  to_shift = 0;
   /* Fetch the argument value.	*/
   if (info->is_long_double && sizeof (long double) > sizeof (double))
     {
@@ -343,7 +333,7 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
      would be really big it could lead to memory problems.  */
   {
     mp_size_t bignum_size = ((ABS (exponent) + BITS_PER_MP_LIMB - 1)
-			     / BITS_PER_MP_LIMB + 4) * sizeof (mp_limb);
+			     / BITS_PER_MP_LIMB + 3) * sizeof (mp_limb);
     frac = (mp_limb *) alloca (bignum_size);
     tmp = (mp_limb *) alloca (bignum_size);
     scale = (mp_limb *) alloca (bignum_size);
@@ -627,24 +617,11 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
       /* All factors but 10^-1 are tested now.	*/
       if (exponent > 0)
 	{
-	  int cnt_l;
-
 	  cy = __mpn_mul_1 (tmp, frac, fracsize, 10);
 	  tmpsize = fracsize;
 	  assert (cy == 0 || tmp[tmpsize - 1] < 20);
 
-	  count_trailing_zeros (cnt_l, tmp[0]);
-	  if (cnt_l < MIN (4, exponent))
-	    {
-	      cy = __mpn_lshift (frac, tmp, tmpsize,
-				 BITS_PER_MP_LIMB - MIN (4, exponent));
-
-	       if (cy != 0)
-		 frac[tmpsize++] = cy;
-	    }
-	  else
-	    (void) __mpn_rshift (frac, tmp, tmpsize, MIN (4, exponent));
-
+	  (void) __mpn_rshift (frac, tmp, tmpsize, MIN (4, exponent));
 	  fracsize = tmpsize;
 	  exp10 |= 1;
 	  assert (frac[fracsize - 1] < 10);
@@ -786,24 +763,13 @@ __printf_fp( FILE *fp, const struct printf_info *info, const **const args )
       {
 	char *tp = cp;
 
-	if (digit == '5' && (*(cp - 1) & 1) == 0)
+	if (digit == '5')
 	  /* This is the critical case.	 */
 	  if (fracsize == 1 && frac[0] == 0)
 	    /* Rest of the number is zero -> round to even.
 	       (IEEE 754-1985 4.1 says this is the default rounding.)  */
-	    goto do_expo;
-	  else if (scalesize == 0)
-	    {
-	      /* Here we have to see whether all limbs are zero since no
-		 normalization happened.  */
-	      size_t lcnt = fracsize;
-	      while (lcnt >= 1 && frac[lcnt - 1] == 0)
-		--lcnt;
-	      if (lcnt == 0)
-		/* Rest of the number is zero -> round to even.
-		   (IEEE 754-1985 4.1 says this is the default rounding.)  */
-		goto do_expo;
-	    }
+	    if ((*(cp - 1) & 1) == 0)
+	      goto do_expo;
 
 	if (fracdig_no > 0)
 	  {
@@ -956,11 +922,7 @@ guess_grouping (unsigned int intdig_max, const char *grouping, wchar_t sepchar)
 
   /* We treat all negative values like CHAR_MAX.  */
 
-#ifdef __CHAR_UNSIGNED__
-  if (*grouping == CHAR_MAX )
-#else
-  if (*grouping == CHAR_MAX || *grouping < 0)
-#endif
+  if (*grouping == CHAR_MAX || *grouping <= 0)
     /* No grouping should be done.  */
     return 0;
 
@@ -970,17 +932,13 @@ guess_grouping (unsigned int intdig_max, const char *grouping, wchar_t sepchar)
       ++groups;
       intdig_max -= *grouping++;
 
-#ifdef __CHAR_UNSIGNED__
-      if (*grouping == CHAR_MAX )
-#else
       if (*grouping == CHAR_MAX || *grouping < 0)
-#endif
 	/* No more grouping should be done.  */
 	break;
       else if (*grouping == 0)
 	{
 	  /* Same grouping repeats.  */
-	  groups += (intdig_max - 1) / grouping[-1];
+	  groups += intdig_max / grouping[-1];
 	  break;
 	}
     }
@@ -1015,11 +973,7 @@ group_number (char *buf, char *bufend, unsigned int intdig_no,
       while (--len > 0);
       *p-- = thousands_sep;
 
-#ifdef __CHAR_UNSIGNED__
-      if (*grouping == CHAR_MAX )
-#else
       if (*grouping == CHAR_MAX || *grouping < 0)
-#endif
 	/* No more grouping should be done.  */
 	break;
       else if (*grouping == 0)
