@@ -4,12 +4,20 @@
 #include <limits.h>
 #include <linux/timex.h>
 
+#ifdef __SVR4_I386_ABI_L1__
+#define adjtimex __adjtimex
+#else
 inline static
 _syscall1(int, adjtimex, struct timex *, ntx);
+#endif
 
 #define MAX_SEC	(LONG_MAX / 1000000L - 2)
 #define MIN_SEC	(LONG_MIN / 1000000L + 2)
 
+#ifndef MOD_OFFSET
+#define modes mode
+#endif
+
 int
 __adjtime(struct timeval * itv, struct timeval * otv)
 {
@@ -28,11 +36,11 @@ __adjtime(struct timeval * itv, struct timeval * otv)
 	return -1;
     }
     tntx.offset = tmp.tv_usec + tmp.tv_sec * 1000000L;
-    tntx.mode = ADJ_OFFSET_SINGLESHOT;
+    tntx.modes = ADJ_OFFSET_SINGLESHOT;
   }
   else
   {
-    tntx.mode = 0;
+    tntx.modes = 0;
   }
   if (adjtimex(&tntx) < 0) return -1;
   if (otv)
