@@ -2,11 +2,15 @@
 #include <sys/socket.h>
 #include <sys/socketcall.h>
 
+#ifdef __SVR4_I386_ABI_L1__
+#define socketcall	__socketcall
+#else
 static inline
 _syscall2(long,socketcall,int,call,unsigned long *,args);
+#endif
 
 int
-accept(int sockfd, struct sockaddr *peer, int *paddrlen)
+accept(int sockfd, const struct sockaddr *peer, int *paddrlen)
 {
 	unsigned long args[3];
 
@@ -15,3 +19,7 @@ accept(int sockfd, struct sockaddr *peer, int *paddrlen)
 	args[2] = (unsigned long)paddrlen;
 	return socketcall(SYS_ACCEPT, args);
 }
+
+#ifdef __ELF__
+#pragma weak accept
+#endif
