@@ -1,12 +1,3 @@
-#ifdef __ELF__
-#ifdef __GNUC__
-struct sockaddr;
-int connect ( int, const struct sockaddr *, int ) __attribute__ (( weak ));
-#else
-#pragma weak connect
-#endif   /* __GNUC__ */
-#endif   /* __ELF__ */
-
 #include <syscall.h>
 #include <sys/socket.h>
 #include <sys/socketcall.h>
@@ -28,3 +19,7 @@ connect(int sockfd, const struct sockaddr *saddr, int addrlen)
 	args[2] = addrlen;
 	return socketcall(SYS_CONNECT, args);
 }
+
+#ifdef __ELF__
+#pragma weak connect
+#endif
