@@ -4,7 +4,7 @@
 
 extern void * ___brk_addr;
 
-extern int __init_brk (void);
+extern int __init_brk ();
 
 int __brk(void * end_data_seg)
 {
