@@ -27,3 +27,8 @@ DEFUN(__drem, (x, y), double x AND double y)
 {
   return ____drem(x, y);
 }
+
+#include <gnu-stabs.h>
+#ifdef weak_alias
+weak_alias (__drem, drem);
+#endif
