@@ -1,7 +1,8 @@
 #include <syscall.h>
 
 #define __KERNEL__ 
-#include <linux/linkage.h>
+#undef asmlinkage
+#define asmlinkage
 #include <sys/shm.h>
 
 int
