@@ -17,3 +17,9 @@ htons(unsigned short int x)
 {
   return __htons (x);
 }
+
+#include <gnu-stabs.h>
+#ifdef elf_alias
+elf_alias (htonl, ntohl);
+elf_alias (htons, ntohs);
+#endif
